@@ -50,6 +50,7 @@ import Control.Applicative
 import Control.Category ((<<<), (>>>))
 import Control.Monad
 import Control.Monad.Catch (MonadCatch(..), MonadThrow(..))
+import qualified Control.Monad.Fail as Fail
 import Control.Monad.Identity
 import Control.Monad.Trans.Class
 import Control.Monad.IO.Class
@@ -129,6 +130,10 @@ instance Monad (FT f m) where
   return = pure
   FT fk >>= f = FT $ \b fr -> fk (\d -> runFT (f d) b fr) fr
 
+instance Fail.MonadFail m => Fail.MonadFail (FT f m) where
+  fail = lift . Fail.fail
+  {-# INLINE fail #-}
+
 instance MonadFree f (FT f m) where
   wrap f = FT (\kp kf -> kf (\ft -> runFT ft kp kf) f)
 
